require "skills_cards_validations/valid"
require "skills_cards_validations/invalid"
require "time"

module SkillsCardsValidations
  module CardValidations
    def id?(id)
      # A 4 to 5 digit number, 0-9.
      # But! No leading 0
      if id_has_leading_zero?(id)
        return Invalid.new(id)
      elsif id.to_s.length == 4 || id.to_s.length == 5
        return Valid.new(id)
      end
      Invalid.new(id)
    end

    def id_has_leading_zero?(id)
      if (id).to_s[0] == "0"
        return true
      end
      return false
    end

    def system?(sy)
      # 1 - 4
      if sy.to_i.between? 1, 4
        return Valid.new(sy)
      end
      Invalid.new(sy)
    end

    def point_range?(pr)
      # 1 - 3
      if pr.to_i.between? 1, 3
        return Valid.new(pr)
      end
      Invalid.new(pr)
    end

    def bond_range?(br)
      # 1 - 4
      if br.to_i.between? 1, 4
        return Valid.new(br)
      end
      Invalid.new(br)
    end

    def card_number?(cn)
      # 1 - 3
      if cn.to_i.between? 1, 3
        return Valid.new(cn)
      end
      Invalid.new(cn)
    end

    def date?(d)
      # 'YYYY-MM-DD'
      begin
        Time.parse(d.to_s)
        return Valid.new(d)
      rescue ArgumentError, NoMethodError
      end
      Invalid.new(d)
    end

    def date_has_valid_month?(d)
      # 'YYYY-MM-DD'
      begin
        month = d.to_s.split('-')[1]
        return month.to_i >= 1 && month.to_i <= 12
      rescue ArgumentError
        return false
      end
    end

    def date_has_valid_day?(d)
      # 'YYYY-MM-DD'
      # Depends on also having a valid month and year!
      begin
        if date_has_valid_month?(d) && date_has_valid_year?(d)
          day = d.to_s.split('-')[2]
          return day.to_i > 0 && day.to_i < 32
        end
      rescue ArgumentError
      end
      return false
    end

    def date_has_valid_year?(d)
      # 'YYYY-MM-DD'
      begin
        year = d.to_s.split('-')[0]
        return year.to_i >= 1000  
      rescue ArgumentError
        return false
      end
    end

    def date_is_after_today?(d)
      # 'YYYY-MM-DD'
      begin
        return Time.parse(d.to_s).to_date > Time.now.to_date
      rescue ArgumentError
        return false
      end
    end

    def date_is_before_last_year?(d)
      # 'YYYY-MM-DD'
      begin
        now = Time.now.to_date
        last_year = Date.civil(now.year-1, now.month, now.day)
        return Time.parse(d.to_s).to_date < last_year
      rescue ArgumentError
        return false
      end
    end

    def privileges?(pr)
      # True or False
      if pr.to_s.downcase == "true" || pr.to_s.downcase == "false"
        return Valid.new(pr)
      end
      Invalid.new(pr)
    end
  end
end